/* ************ COPYRIGHT 2011, H. G. Katzgraber. ********************* */
/*                                                                      */
/* EX 3:        Simple sample Monte Carlo estimate of int f(x) = x^EXP  */
/*                                                                      */
/*              Computes up to 10^N log-space random numbers and prints */
/*              integral estimate with deviation from exact             */
/*		Exact value of the integral is I[f(x)] = 1/(EXP + 1)	*/
/*		Chage EXP and see how convergence changes for large EXP	*/
/* INSTR:       Compile with Makefile via 'make', execute with ./runme  */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "r1279.h"
#include "nrutil.h"		/* helper routines for array allocation	*/

#define N	7           	/* number of 10^N random nums    	*/
#define EXP	3.0		/* exponent of power-law function	*/

long seedgen(void);

int main()
{
    int 	n, k, m, i;
    double 	x, integral, exact;
    long 	seed;
    
    seed        = seedgen();    /* have seedgen compute a random seed   */
    setr1279(seed);             /* seed the genertor                    */

    exact	= 1.0/(((double) EXP) + 1);

    printf("exact value = %f\n\n",exact);
    printf("MCS     Integral Error\n");

    for(k = 1; k <= N; k++){
	m = pow(10,k);
	integral = 0.0;
	for(n = 1; n <= m; n++){
	    x = r1279();
	    integral += pow(x,EXP);
	}
	integral /= ((double) m);
	printf("%2.1e %f %6.5e\n",
		((double) m),
		integral,
		fabs(integral - exact)
		);
    }
    
    return EXIT_SUCCESS;
}
